'''
    xam.addon
    ---------

    Contains the Addon class to represent an XBMC addon

    :copyright: (c) 2012 Jonathan Beluch
    :license: BSD, see LICENSE for more details.

'''
from functools import wraps
from xml.etree import ElementTree as ET
try:
    from collections import OrderedDict
except ImportError:
    from collective.ordereddict import OrderedDict
from .common import UnicodeBuilder


def silence_attr_error(func):
    '''A decorator to allow a raised AttributeError to pass silently'''
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except AttributeError:
            return None
    return wrapper


# TODO: Extensions should parsed agnostically, and return an ordered dict
#       of extensions, queryable by the "point" attribute.
#       * A shortcut to metadata is acceptable

# TODO: For each extension, we need a list of attrs and child tags that
#       should be queryable.
class Addon(object):
    '''A class to hold information regarding an XBMC addon.'''

    def __init__(self, xml):
        '''Intializes an addon object for the provided addon xml'''
        self._xml = xml
        # These three properties are required, everything else is optional
        required_attrs = ['id', 'name', 'version']
        for attr in required_attrs:
            if attr not in self.xml.keys():
                raise Exception('Provided xml is missing the %s attribute' % attr)

    @classmethod
    def from_filename(cls, filename):
        xml = ET.parse(filename, parser=UnicodeBuilder())
        return cls(xml.getroot())

    def __repr__(self):
        return '<Addon %s %s>' % (self.id, self.version)

    @property
    def xml(self):
        '''Returns the root xml element for the addon'''
        return self._xml

    def to_xml_string(self):
        '''Returns a string containing the addon's xml'''
        return ET.tostring(self.xml, encoding='utf-8')

    def _get_version(self):
        '''Returns the addon's version'''
        return self.xml.get('version')

    def _set_version(self, version):
        '''Sets the addon's version'''
        self.xml.set('version', version)

    version = property(_get_version, _set_version)
    del _get_version, _set_version

    @property
    def id(self):
        '''Returns the addon's id'''
        return self.xml.get('id')

    @property
    def name(self):
        '''Returns the addon's name'''
        return self.xml.get('name')

    @property
    def provider(self):
        '''The addon's provider'''
        return self.xml.get('provider-name')

    @property
    def dependencies(self):
        '''A dict of required addons this addon is dependent on. Key is
        addon id and value is the required version.'''
        requires = self.xml.find('requires')
        if requires is not None:
            return OrderedDict((required.get('addon'), required.get('version'))
                    for required in requires.findall('import'))
        return {}

    def set_dependency_version(self, addon_id, addon_version):
        '''Sets the version attribute of an addon dependency.

        :param addon_id: The id of the addon dependency
        :param addon_version: The new version string
        '''
        addon = self.xml.find('./requires/import[@addon="%s"]' % addon_id)
        if addon is not None:
            addon.set('version', addon_version)

    @property
    def extensions(self):
        '''Returns a dict of extension xml nodes keyed by the 'point'
        attribute.
        '''
        extensions = self.xml.findall('extension')
        if extensions:
            return dict((ext.get('point'), ext) for ext in extensions)

    @property
    @silence_attr_error
    def metadata(self):
        '''The addon's metadata xml element'''
        try:
            return self.xml.find('./extension[@point="xbmc.addon.metadata"]')
        except SyntaxError:
            # ElementTree < 1.3 (python < 2.7) doesn't include xpath support
            for tag in self.xml.findall('extension'):
                if tag.get('point') == 'xbmc.addon.metadata':
                    return tag
            return None

    #@property
    #@silence_attr_error
    #def provides(self):
        #return self.pluginsource.find('provides').text

    @property
    @silence_attr_error
    def languages(self):
        '''Returns a list of language codes the addon provides content for.'''
        try:
            return self.metadata.find('language').text.split()
        except AttributeError:
            # no lang tag, self closing lang tag, or empty lang tag
            return []


    @property
    @silence_attr_error
    def platform(self):
        '''The addon's platform'''
        return self.metadata.find('platform').text

    @property
    @silence_attr_error
    def summaries(self):
        '''Returns an OrderedDict of the addon's summaries. Key is an
        XBMC language code and value is the corresponding summary text.
        '''
        # TODO: figure out if we can default to en when no lang attr is present
        return OrderedDict((sum.get('lang'), sum.text) for sum in
                            self.metadata.findall('summary'))

    @silence_attr_error
    def summary(self, lang=None):
        '''Returns the summary for the provided language code or the
        first summary available if no language is provided. None is
        returned if there are no summaries available.
        '''
        if not lang:
            try:
                return self.summaries.values()[0]
            except IndexError:
                return None
        return self.summaries.get(lang)

    @property
    @silence_attr_error
    def descriptions(self):
        '''Returns an OrderedDict of the addon's descriptions. Key is
        an XBMC language code and value is the corresponding
        description text.'''
        return OrderedDict((sum.get('lang'), sum.text) for sum in
                            self.metadata.findall('description'))

    @silence_attr_error
    def description(self, lang=None):
        '''Returns the desciption for the provided language code or the
        first desciption available if no language is provided. None is
        returned if there are no descriptions available.
        '''
        if not lang:
            try:
                return self.descriptions.values()[0]
            except IndexError:
                return None
        return self.descriptions.get(lang)

    def to_dict(self):
        return {
            'id': self.id,
            'name': self.name,
            'version': self.version,
            'provider': self.provider,
            'dependencies': self.dependencies,
            'summaries': self.summaries,
            'descriptions': self.descriptions,
            'platform': self.platform,
            '_xml': ET.tostring(self.xml),
        }







'''
    xam.repos
    ---------

    Contains URLs for the official XBMC addon repositories.

    Variable names surface as command line arguments to the --repo
    option. Each tuple contains the url to the addons.xml, the data dir
    url and the checksum url.

    :copyright: (c) 2012 Jonathan Beluch
    :license: BSD, see LICENSE for more details.
'''


FRODO= (
    'http://mirrors.xbmc.org/addons/frodo/addons.xml',
    'http://mirrors.xbmc.org/addons/frodo/',
    'http://mirrors.xbmc.org/addons/frodo/addons.xml.md5',
)

FRODO_PRE = (
    'http://mirrors.xbmc.org/addons/frodo-pre/addons.xml',
    'http://mirrors.xbmc.org/addons/frodo-pre/',
    'http://mirrors.xbmc.org/addons/frodo-pre/addons.xml.md5',
)

DHARMA = (
    'http://mirrors.xbmc.org/addons/dharma/addons.xml',
    'http://mirrors.xbmc.org/addons/dharma/',
    'http://mirrors.xbmc.org/addons/dharma/addons.xml.md5',
)

DHARMA_PRE = (
    'http://mirrors.xbmc.org/addons/dharma-pre/addons.xml',
    'http://mirrors.xbmc.org/addons/dharma-pre/',
    'http://mirrors.xbmc.org/addons/dharma-pre/addons.xml.md5',
)

EDEN = (
    'http://mirrors.xbmc.org/addons/eden/addons.xml',
    'http://mirrors.xbmc.org/addons/eden/',
    'http://mirrors.xbmc.org/addons/eden/addons.xml.md5',
)

EDEN_PRE = (
    'http://mirrors.xbmc.org/addons/eden-pre/addons.xml',
    'http://mirrors.xbmc.org/addons/eden-pre/',
    'http://mirrors.xbmc.org/addons/eden-pre/addons.xml.md5',
)



'''
    xam.xam
    ----------

    This module is the entry point for the console script xam.

    :copyright: (c) 2012 Jonathan Beluch
    :license: BSD, see LICENSE for more details.
'''


USAGE = \
'''
    %(prog)s [options] {all, depends, get, info, search} [arg [arg ...]]

synopsis:
    %(prog)s all
    %(prog)s depends <plugin_id>
    %(prog)s get <plugin_id> [<plugin_id> ...]
    %(prog)s info <plugin_id>
    %(prog)s search <search_text>

commands:
    all
        Prints addon ids and current version numbers for every addon in the
        repo.

    depends
        Prints addon ids and current version for every addon which lists
        the specified addon id as a dependency.

    get
        Downloads the addon zip file and any assets to a folder in the
        current working directory. The folder name will be the addon id.

        NOTE: At this time, the download functionality will only work
        properly for repositories which zip their addons. Using this
        command for a repository which doesn't zip addons will definitely
        not download all of the necessary addon files.

    info
        Prints the specified addon's xml file to STDOUT.

    search
        Prints addon ids and matching lines for the provided text. A case
        insensitive seach of addon xml files is performed.

optional arguments:
    -r URL, --repo URL
        Either an official repo name [eden, eden_pre, dharma, dharma_pre]
        or a URL to a zipped repository file. Defaults to "eden". A list
        of 3rd party repos can be found here:
        http://wiki.xbmc.org/index.php?title=3rd_party_add-on_repositories

examples:
    %(prog)s --repo dharma all
        List addon id and version for every addon in the Dharma repo.

    %(prog)s --repo http://bluecop-xbmc-repo.googlecode.com/files/repository.bluecop.xbmc-plugins.zip all
        List addon id and version for every addon in bluecop's repo.

    %(prog)s depends script.module.xbmcswift
        List addon ids and version for every addon which lists
        script.module.xbmcswift as a dependency.

    %(prog)s get plugin.video.academicearth plugin.video.khanacademy
        Download the current versions of Acacdemic Earth and Khan Academy.

    %(prog)s info plugin.video.classiccinema
        Print addon.xml for the current Classic Cinema plugin.

    %(prog)s search lecture
        Print addon ids and matching lines for any addon containing the
        word lecture.
'''



'''
    xam.common
    ----------

    Contains some helpful functions with no other home.

    :copyright: (c) 2012 Jonathan Beluch
    :license: BSD, see LICENSE for more details.

'''
from xml.etree import ElementTree as ET
import requests


class UnicodeBuilder(ET.XMLTreeBuilder):

    def _fixtext(self, text):
        '''Override the parent class which attempts to encode to ascii'''
        return text


def urlretrieve(url, filename):
    '''Downloads the resource found at the remote url to the provided
    filename if the url returns an OK status.'''
    print '* Downloading %s to %s' % (url, filename)
    req = requests.get(url)
    if req.status_code == requests.codes.ok:
        with open(filename, 'wb') as output:
            output.write(req.content)



import os
import logging
from urllib import urlretrieve
from cliff.command import Command
from xam import repos
from xam.repository import get_repo


REPO_NAMES = [attr for attr in repos.__dict__.keys()
              if not attr.startswith('_')]


def uppercase(inp):
    return inp.upper()


def add_repo_arg(parser):
    parser.add_argument('--repo', type=uppercase, choices=REPO_NAMES,
                        default='FRODO')


def generate_addon_output(addon):
    title = '%s (%s %s)' % (addon.name, addon.id, addon.version)
    lines = [
        title,
        '-' * len(title),
        addon.to_xml_string(),
    ]
    return '\n'.join(lines)


class ListAddons(Command):
    '''List addon id and addon version for every addon in the provided
    repo.
    '''

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(ListAddons, self).get_parser(prog_name)
        add_repo_arg(parser)
        return parser

    def take_action(self, parsed_args):
        self.log.debug('Showing addon ids and versions for %s repo.'
                      % parsed_args.repo)

        repo = get_repo(parsed_args.repo)
        for addon in sorted(repo.addons, key=lambda addon: addon.id):
            self.app.stdout.write('%s %s\n' % (addon.id, addon.version))



class ShowAddonInfo(Command):
    '''Shows the contents of addon.xml for the provided addon id.'''

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(ShowAddonInfo, self).get_parser(prog_name)
        add_repo_arg(parser)
        parser.add_argument('addon_id')
        return parser

    def take_action(self, parsed_args):
        addonid = parsed_args.addon_id
        reponame = parsed_args.repo

        self.log.debug('Showing info for %s in %s' % (addonid, reponame))

        repo = get_repo(reponame)
        try:
            addon = (addon for addon in repo.addons
                     if addon.id == addonid).next()
        except StopIteration:
            raise RuntimeError('No addon found with id %s' % addonid)

        self.app.stdout.write(generate_addon_output(addon))



class ShowDependentAddons(Command):
    '''Returns a list of addon id/versions which list the specified
    addon as a dependency.
    '''

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(ShowDependentAddons, self).get_parser(prog_name)
        add_repo_arg(parser)
        parser.add_argument('addon_id')
        return parser

    def take_action(self, parsed_args):
        addonid = parsed_args.addon_id
        reponame = parsed_args.repo

        self.log.debug('Listing dependent addons for info for %s in %s'
                       % (addonid, reponame))

        repo = get_repo(reponame)
        dependents = [addon for addon in repo.addons
                      if addonid in addon.dependencies.keys()]

        for addon in dependents:
            self.app.stdout.write('%s %s\n' % (addon.id, addon.version))


class GetAddon(Command):
    '''For the provided addon ids, attempts to download the zipped
    addon, fanart.jpg, icon.png and changelog.txt. All downloaded files
    are located in a directory within the current working directory
    named after the addon id.
    '''

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(GetAddon, self).get_parser(prog_name)
        add_repo_arg(parser)
        parser.add_argument('addon_id', nargs='+')
        return parser

    def take_action(self, parsed_args):
        addonids = parsed_args.addon_id
        reponame = parsed_args.repo

        repo = get_repo(reponame)
        addons = [addon for addon in repo.addons if addon.id in addonids]
        data_urls = [repo.addon_data_urls(addon) for addon in addons]

        for addon, urls in zip(addons, data_urls):
            addonids.remove(addon.id)
            self.log.info('Downloading %s from %s to %s'
                          % (addon.id, reponame, addon.id))

            try:
                os.mkdir(addon.id)
            except OSError:
                raise RuntimeError('* Cannot make directory %s. If it exists, '
                                   'try removing it and running again.'
                                   % addon.id)

            for _, url in urls.items():
                filename = os.path.join(addon.id, url.rsplit('/', 1)[1])
                self.log.debug('Downloading %s to %s' % (url, filename))
                urlretrieve(url, filename)

        for addonid in addonids:
            # we couldn't find an addon with that id
            self.log.warning("Couldn't find %s in %s" % (addonid, reponame))



class SearchAddons(Command):
    '''Does a case insensitive (and xml ignorant) search of all addons'
    addon.xml file and lists matches.
    '''

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(SearchAddons, self).get_parser(prog_name)
        add_repo_arg(parser)
        parser.add_argument('search_term')
        return parser

    def take_action(self, parsed_args):
        reponame = parsed_args.repo
        search_term = parsed_args.search_term

        self.log.debug('Searching for %s in %s' % (search_term, reponame))

        repo = get_repo(reponame)
        text = search_term.lower()

        for addon in repo.addons:
            if text in addon.to_xml_string():
                lines = addon.to_xml_string().splitlines()
                matching_lines = [line for line in lines if text in line.lower()]
                for i, line in enumerate(matching_lines):
                    self.app.stdout.write('%s:%d: %s\n' % (addon.id, i + 1,
                                                         line))



'''
    xam.repository
    --------------

    Contains the Repository class and some helper functions.
'''
import os
import sys
import base64
import logging
from zipfile import ZipFile
from urlparse import urljoin
from xml.etree import ElementTree as ET
import requests

import repos
from .common import urlretrieve, UnicodeBuilder
from .addon import Addon


def get(url):
    '''Returns a response for the given url.'''
    return requests.get(url).content


def safe_cache_fn(key):
    '''Returns an absolute path name for a cached file based on the
    provided key. The same file will always be returned for a given
    key.
    '''
    return cache_fn(base64.urlsafe_b64encode(key))


def cache_fn(filename):
    '''Returns an absolute filename in the xam cache folder for the
    given base filename.
    '''
    return os.path.join(os.getenv('HOME'), '.xam_cache', filename)


def write_file(filename, content):
    '''Writes content to filename'''
    with open(filename, 'w') as fileobj:
        fileobj.write(content)


def get_repo(name_or_url):
    '''Returns a repository for a given name or url. name_or_url can be
    an official repository name found in repos.py or it can be a url to
    a zipped repository file.
    '''
    if hasattr(repos, name_or_url.upper()):
        return Repository(*getattr(repos, name_or_url.upper()))
    else:
        return Repository.from_zip(name_or_url)


class Repository(object):
    '''A class representing an XBMC addon repository.'''

    log = logging.getLogger(__name__)

    def __init__(self, info_url, datadir_url, checksum_url=None):
        '''If checksum_url is None, the remote addons.xml will be
        checked for every request. If provided, checksums will be
        verified to see if an update has occured.
        '''
        self.info_url = info_url
        self.checksum_url = checksum_url
        self.datadir_url = datadir_url
        if not self.datadir_url.endswith('/'):
            self.datadir_url = self.datadir_url + '/'
        self._remote_md5 = None
        self._local_md5 = None
        self._addons_xml = None
        self._addons = []
        self.xml = None
        self.parse_addons()

    @classmethod
    def from_zip(cls, zip_url):
        '''Returns a Repository instance for the provided zip_url.
        zip_url should be a url to a zipped repository file.
        '''
        # TODO: Download zip file to a temp location so it will be cleared
        filename = safe_cache_fn(zip_url)
        urlretrieve(zip_url, filename)

        # Attempt to extract the content addon.xml within the zip file
        zipfile = ZipFile(filename)
        try:
            addon_xml_filename = (filename for filename in zipfile.namelist()
                                  if filename.endswith('addon.xml')).next()
        except StopIteration:
            sys.exit('This repository doesn\'t appear to contain an addon.xml'
                     ' file.')

        # Parse the addon.xml and extract the three URLs
        with zipfile.open(addon_xml_filename) as addon_xml:
            xml = ET.parse(addon_xml, parser=UnicodeBuilder())
        extension = xml.find('extension[@point="xbmc.addon.repository"]')
        info_url = extension.find('info').text
        checksum_url = extension.find('checksum').text
        datadir_url = extension.find('datadir').text

        # TODO: Add support for non-zipped addons
        if extension.find('datadir').get('zip') != True:
            self.log.warning('Repositories which do not zip addons are '
                   'unsupported at this time. The download functionality might'
                   ' not work properly.')
        return cls(info_url, datadir_url, checksum_url)

    @property
    def addons_xml(self):
        '''Returns the current version of the repository's addons.xml
        as a string. If this repository has a checksum_url, we will
        attempt to use a cached version of addons.xml if the checksum
        is still valid.
        '''
        if self.remote_md5 is None or self.local_md5 != self.remote_md5:
            # Download everything
            self.log.debug('* Updating addons.xml from remote...')
            self._addons_xml = get(self.info_url)
            if self.checksum_url is not None:
                # Some repos don't have a checksum URL, so we will have to
                # download them everytime
                write_file(safe_cache_fn(self.info_url), self._addons_xml)
                write_file(safe_cache_fn(self.checksum_url), self.remote_md5)
        elif self._addons_xml is None:
            self.log.debug( '* Local addons.xml is up to date...')
            with open(safe_cache_fn(self.info_url)) as addons_file:
                self._addons_xml = addons_file.read()
        return self._addons_xml

    @property
    def remote_md5(self):
        '''Returns the remote md5 checksum for a repository or None if
        the repository doesn't have a checksum url.
        '''
        if self.checksum_url is None:
            return None
        if self._remote_md5 is None:
            self._remote_md5 = get(self.checksum_url)
        return self._remote_md5

    @property
    def local_md5(self):
        '''Returns the md5 checksum of the local cached addons.xml or
        None if there is no cached copy.
        '''
        if self._local_md5 is None:
            try:
                with open(safe_cache_fn(self.checksum_url)) as checksum_file:
                    self._local_md5 = checksum_file.read()
            except IOError:
                return None
        return self._local_md5

    @property
    def addons(self):
        '''Returns a list of Addons for this repository'''
        return self._addons

    def parse_addons(self):
        '''Parses this repository's addons.xml file and creates Addon
        instances for each addon listed.'''
        self.xml = ET.fromstring(self.addons_xml, parser=UnicodeBuilder())
        self._addons = [Addon(addon) for addon in self.xml.findall('addon')]

    def addon_data_urls(self, addon):
        '''Returns a dict of urls for the provided addons assets.

        Returned dict's keys are ['zip', 'icon', 'fanart', 'changelog']
        '''
        addon_dir = urljoin(self.datadir_url, '%s/' % addon.id)
        return {
            'zip': urljoin(addon_dir, '%s-%s.zip' % (addon.id, addon.version)),
            'icon': urljoin(addon_dir, 'icon.png'),
            'fanart': urljoin(addon_dir, 'fanart.jpg'),
            'changelog': urljoin(addon_dir,
                                 'changelog-%s.txt' % addon.version),
        }



import os
import sys
import logging
from cliff.app import App
from cliff.commandmanager import CommandManager


class XAM(App):

    log = logging.getLogger(__name__)

    #CONSOLE_MESSAGE_FORMAT = App.LOG_FILE_MESSAGE_FORMAT

    def __init__(self):
        super(XAM, self).__init__(
            description='XBMC Addon Manager',
            version='0.1',
            command_manager=CommandManager('xam'),
            )

    def initialize_app(self, argv):
        self.log.debug('initialize_app')

        # Set up .xam_cache folder
        try:
            os.mkdir(os.path.join(os.getenv('HOME'), '.xam_cache'))
        except OSError:
            pass

    def prepare_to_run_command(self, cmd):
        self.log.debug('prepare_to_run_command %s', cmd.__class__.__name__)

    def clean_up(self, cmd, result, err):
        self.log.debug('clean_up %s', cmd.__class__.__name__)
        if err:
            self.log.debug('got an error: %s', err)


def main(argv=sys.argv[1:]):
    myapp = XAM()
    return myapp.run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



'''
    xam.extensions
    --------------

    Contains an up to date list of XBMC extension points and their
    respective attributes and child tag names.

    :copyright: (c) 2012 Jonathan Beluch
    :license: BSD, see LICENSE for more details.
'''


EXTENSIONS = {
    # extension_name: ([extension_attrs], [extension_children_tag_names])
    'xbmc.gui.skin': ([], []),
    'xbmc.gui.webinterface': ([], []),
    'xbmc.addon.repository': ([], []),
    'xbmc.service': (['library', 'start'], []),
    'xbmc.metadata.scraper.albums': ([], []),
    'xbmc.metadata.scraper.artists': ([], []),
    'xbmc.metadata.scraper.movies': ([], []),
    'xbmc.metadata.scraper.musicvideos': ([], []),
    'xbmc.metadata.scraper.tvshows': ([], []),
    'xbmc.metadata.scraper.library': ([], []),
    'xbmc.ui.screensaver': ([], []),
    'xbmc.player.musicviz': ([], []),
    'xbmc.python.pluginsource': (['library'], []),
    'xbmc.python.script': ([], []),
    'xbmc.python.weather': ([], []),
    'xbmc.python.subtitles': ([], []),
    'xbmc.python.lyrics': ([], []),
    'xbmc.python.library': ([], []),
    'xbmc.python.module': ([], []),
    'xbmc.addon.video': ([], []),
    'xbmc.addon.audio': ([], []),
    'xbmc.addon.image': ([], []),
}



import os
from unittest import TestCase
from xml.etree import ElementTree as ET
from xam import Addon
try:
    from collections import OrderedDict
except ImportError:
    from collective.ordereddict import OrderedDict


class TestAddon(TestCase):

    def assert_attrs(self, obj, attrs):
        for attr_name, expected_value in attrs.items():
            attr_value = getattr(obj, attr_name)
            self.assertEqual(expected_value, attr_value)
            self.assertTrue(isinstance(attr_value, unicode))

    def assert_dict(self, expected, actual):
        for key, val in actual.items():
            self.assertTrue(isinstance(key, unicode))
            self.assertTrue(isinstance(val, unicode))
        self.assertEqual(expected, actual)

    def test_parse(self):
        addon = Addon.from_filename(os.path.join(os.path.dirname(__file__), 'data', 'addon.xml'))

        expected = {
            # attr_name: expected_value
            'version': u'1.2.1',
            'id': u'plugin.video.academicearth',
            'name': u'Academic Earth',
            'provider': u'Jonathan Beluch (jbel)',
        }
        self.assert_attrs(addon, expected)

        self.assert_dict({
            u'xbmc.python': u'2.0',
            u'script.module.beautifulsoup': u'3.0.8',
            u'script.module.xbmcswift': u'0.2.0',
            u'plugin.video.youtube': u'2.9.1',
        }, addon.dependencies)

        self.assertEqual(addon.languages, ['en', 'fr'])
        self.assertNotEqual(None, addon.metadata)
        self.assertEqual('all', addon.platform)
        self.assertEqual(OrderedDict(
            [(None, 'Watch lectures from Academic Earth (http://academicearth.org)')]
        ), addon.summaries)
        self.assertEqual('Watch lectures from Academic Earth (http://academicearth.org)',
                         addon.summary())
        #self.assertEqual('Watch lectures from Academic Earth (http://academicearth.org)',
                         #addon.summary('en'))
        self.assertEqual(OrderedDict(
            [(None,'Browse online courses and lectures from the world\'s top scholars.')]
        ), addon.descriptions)
        self.assertEqual('Browse online courses and lectures from the world\'s top scholars.',
                         addon.description())


    def test_setters(self):
        xml = ET.parse(os.path.join(os.path.dirname(__file__), 'data', 'addon.xml')).getroot()
        addon = Addon(xml)
        self.assertEqual('1.2.1', addon.version)
        addon.version = '1.2.2'
        self.assertEqual('1.2.2', addon.version)


    def test_to_dict(self):
        addon = Addon.from_filename(os.path.join(os.path.dirname(__file__), 'data', 'addon.xml'))
        actual = addon.to_dict()

        with open(os.path.join(os.path.dirname(__file__), 'data', 'addon.xml')) as inp:
            xml = inp.read()
        expected = {
            'id': u'plugin.video.academicearth',
            'name': u'Academic Earth',
            'version': u'1.2.1',
            'provider': u'Jonathan Beluch (jbel)',
            'dependencies': {
                'xbmc.python': '2.0',
                'script.module.beautifulsoup': '3.0.8',
                'script.module.xbmcswift': '0.2.0',
                'plugin.video.youtube': '2.9.1',
            },
            'summaries': {None: u"Watch lectures from Academic Earth (http://academicearth.org)"},
            'descriptions': {None: u"Browse online courses and lectures from the world's top scholars."},
            'platform': 'all',
            '_xml': xml,
        }

        for key, val in expected.items():
            if not key.startswith('_'):
                self.assertEqual(val, actual[key])


LANG_XML_TMP = '''
<addon id="plugin.video.academicearth" name="Academic Earth" provider-name="Jonathan Beluch (jbel)" version="1.2.1">
  <extension point="xbmc.addon.metadata">
    %s
  </extension>
</addon>
'''
class TestLangTags(TestCase):

    def test_no_lang_tag(self):
        xmlstr = LANG_XML_TMP % ''
        addon = Addon(ET.fromstring(xmlstr))
        self.assertEqual(addon.languages, [])

    def test_self_close_lang_tag(self):
        xmlstr = LANG_XML_TMP % '<language/>'
        addon = Addon(ET.fromstring(xmlstr))
        self.assertEqual(addon.languages, [])

    def test_empty_lang_tag(self):
        xmlstr = LANG_XML_TMP % '<language></language>'
        addon = Addon(ET.fromstring(xmlstr))
        self.assertEqual(addon.languages, [])

    def test_data_lang_tag(self):
        xmlstr = LANG_XML_TMP % '<language>en</language>'
        addon = Addon(ET.fromstring(xmlstr))
        self.assertEqual(addon.languages, ['en'])

        xmlstr = LANG_XML_TMP % '<language>en fr</language>'
        addon = Addon(ET.fromstring(xmlstr))
        self.assertEqual(addon.languages, ['en', 'fr'])



if __name__ == '__main__':
   unittest.main()



import unittest
try:
    from collections import OrderedDict
except ImportError:
    from collective.ordereddict import OrderedDict
from xam.cli.release import compare_versions


class TestRelease(unittest.TestCase):

    def test_compare_versions(self):
        known_values = [
            (('1.0', '1.1'), -1),
            (('1.1', '0.4'), 1),
            (('1.0', '1.0'), 0),
            (('0.0.1', '.1'), 0),
            (('0.0.0', '0'), 0),
            (('1', '0'), 1),
            (('.12', '.2'), 1),
            (('1.3', '1.3'), 0),
        ]
        for inp, expected in known_values:
            self.assertEqual(compare_versions(*inp), expected)


if __name__ == '__main__':
    unittest.main()



'''
    xam.release
    -----------

    Contains the release script to automate the release process for an XBMC
    addon.

    :copyright: (c) 2012 Jonathan Beluch
    :license: BSD, see LICENSE for more details.
'''
import re
import os
import sys
from subprocess import check_call, check_output
import logging

from cliff.command import Command
from colorama import init
init(strip=not sys.stdout.isatty()) # strip colors if stdout is redirected
from termcolor import cprint, colored

from xam.addon import Addon
from xam.repository import get_repo
from xam.cli import REPO_NAMES, add_repo_arg




BLUE = lambda text: colored(text, 'blue')
GREEN = lambda text: colored(text, 'green')
RED = lambda text: colored(text, 'red')

def bump_minor(version_str):
    '''Given a version string, increments the right-most number by 1 and
    returns the new value.

    >>> bump_minor('1.2.3')
    '1.2.4'
    '''
    left, right = version_str.rsplit('.', 1)
    right = int(right) + 1
    return '%s.%s' % (left, right)


def compare_versions(version_a, version_b):
    '''Compares two versions strings and returns 1 if version_a is greater, -1
    is version_b is greater or 0 if they are equal.

    >>> compare_versions('1.0', '1.1')
    -1
    >>> compare_versions('1.1', '0.4')
    1
    >>> compare_versions('0.0.1', '.1')
    0
    '''
    if version_a == version_b:
        return 0

    # Convert parts to ints and strip out blank parts:
    # '.1'.split('.') will result in ['', '1']
    aparts = [int(part) for part in version_a.split('.') if part]
    bparts = [int(part) for part in version_b.split('.') if part]

    # if part lengths are unequal prepend with zeros
    if len(aparts) > len(bparts):
        bparts = [0] * (len(aparts) - len(bparts)) + bparts
    elif len(bparts) > len(aparts):
        aparts = [0] * (len(bparts) - len(aparts)) + aparts

    # same number of parts now, start comparing from leftmost part
    for a, b in zip(aparts, bparts):
        if a > b:
            return 1
        elif b > a:
            return -1
    # the only way we can get here, is if the versions strings aren't equal,
    # but when prepended with zeros they are equal. e.g.: .1 and 0.0.1
    return 0


def write_file(path, contents):
    '''Writes the given contents to the given path'''
    with open(path, 'w') as out:
        out.write(contents)


def generate_email_output(addon_id, version, git_url, tag, xbmc_version):
    '''Prints the message body for a mailing list email.'''
    lines = [
        '',
        '+--------------------+',
        '| Mailing List Email |',
        '+--------------------+',
        '',
        'To: xbmc-addons@lists.sourceforge.net',
        'Subject: [git pull] %s' % addon_id,
        '',
        '*addon - %s' % addon_id,
        '*version - %s' % version,
        '*url - %s' % git_url,
        '*tag - %s' % tag,
        '*xbmc version - %s' % xbmc_version,
        '',
        '',
    ]
    return lines


class ReleaseAddon(Command):
    '''Release a new verison of an addon'''

    log = logging.getLogger(__name__)

    def exit_if_negative(self, msg):
        if not self.yes_no(msg):
            raise RuntimeError('Aborting.')

    def yes_no(self, msg):
        '''Returns False if user entered n or N, True otherwise'''
        ans = raw_input(msg + ' (Y/n) ')
        return ans not in ['N', 'n']
        

    def prompt(self, msg, default=None):
        '''Prints a message and gathers input from the user. The provided default
        is returned if the user provides no input.
        '''
        if default is not None:
            msg = '%s [%s] ' % (msg, default)
        else:
            msg += ': '
        ans = raw_input(msg)
        if ans is not None and len(ans) > 0:
            return ans
        return default

    def get_parser(self, prog_name):
        parser = super(ReleaseAddon, self).get_parser(prog_name)
        add_repo_arg(parser)
        return parser

    def get_cwd_addon(self):
        '''Returns a xam.Addon object for the current addon. Aborts the script
        if no addon.xml file can be found in the current working directory.
        '''
        try:
            return Addon.from_filename('addon.xml')
        except IOError:
            raise RuntimeError('Could not find addon.xml in the current '
                               'directory. Run this script from the root '
                               'directory of your addon.')

    def update_dependencies(self, addon_to_release, xbmc_version):
        '''For any required dependencies, attempts to to update the version number
        to the newest version available in the XBMC official repository.
        '''
        repo = get_repo(xbmc_version)
        addons = repo.addons
        for addon_id, addon_version in addon_to_release.dependencies.items():
            if addon_id != 'xbmc.python':  # skip python dependency
                try:
                    addon = (addon for addon in repo.addons if addon.id == addon_id).next()
                except StopIteration:
                    # ERROR: no addon with that id
                    sys.exit('You have a dependency listed on "%s" but I can\'t '
                             'find an addon with that ID in the %s repository. '
                             'Aborting.' % (addon_id, xbmc_version))

                if compare_versions(addon_version, addon.version) > 0:
                    # ERROR: newest version available is < specified version
                    sys.exit('You require version %s for addon %s, but the newest '
                             'version I can find in the repository is %s. '
                             'Aborting.' % (addon_version, addon_id,
                                            addon.version))
                elif compare_versions(addon_version, addon.version) < 0:
                    # found newer version, prompt for update
                    msg = ('[?] There is a newer version of %s available. '
                           'Would you like to update the dependency from %s to'
                           ' %s?' % (BLUE(addon_id), BLUE(addon_version),
                                     BLUE(addon.version)))
                    if self.yes_no(msg):
                        self.app.stdout.write('Writing to addon.xml...')
                        addon_to_release.set_dependency_version(addon_id, addon.version)
                        write_file('addon.xml', addon_to_release.to_xml_string())
                        self.app.stdout.write(GREEN('OK') + '\n')
                else:
                    msg = 'Dependency %s is already at the newest version.\n' % BLUE(addon_id)
                    self.app.stdout.write(msg)

    def take_action(self, parsed_args):
        '''Performs a release of an XBMC addon.'''
        # Parse addon.xml
        addon = self.get_cwd_addon()
        reponame = parsed_args.repo

        # determine local git branch
        current_branch = check_output('git symbolic-ref HEAD 2>/dev/null',
                                      shell=True)
        if current_branch is None or not current_branch.startswith('refs/heads/'):
            raise RuntimeError('Could not determine current git branch.')
        current_branch_trimmed = current_branch.replace('refs/heads/', '').strip()

        lines = [
            '',
            '''  oooo    ooo  .oooo.   ooo. .oo.  .oo.''',
            '''   `88b..8P'  `P  )88b  `888P"Y88bP"Y88b''',
            '''     Y888'     .oP"888   888   888   888''',
            '''   .o8"'88b   d8(  888   888   888   888''',
            '''  o88'   888o `Y888""8o o888o o888o o888o''',
            '',
            '''           (XBMC ADDON MANAGER)''',
            '',
            '  I\'m going to help you create a new release for %s.' % BLUE(addon.name),
            '',
            '.. Init ..',
            '',
            'Please answer the following:',
            '',
        ]   
        self.app.stdout.write('\n'.join(lines))
        
        # Verify current branch
        msg = '[?] I see you are using git. Is %s the branch you want to use for the release?' % BLUE(current_branch_trimmed)
        self.exit_if_negative(msg)

        # Verify xbmc_version
        msg = '[?] This release will be for XBMC %s. Is this correct?' % BLUE(reponame)
        self.exit_if_negative(msg)

        # Check for dirty repo
        is_dirty = check_output('git status --porcelain', shell=True)
        if is_dirty:
            msg = ('[?] Your git working directory has uncommitted changes. '
                   'These changes will be included in the release commit. '
                   'Would you like to continue anyway?')
            self.exit_if_negative(msg)

        # TODO
        '  * You are using a `git remote` named "origin".',

        # Now onto the release setup
        lines = [
            '',
            '.. Release Metadata for %s ..' % BLUE(addon.id),
            '',
        ]
        self.app.stdout.write('\n'.join(lines) + '\n')

        # version
        msg = '[?] The current version is %s. What should the new version be?' % BLUE(addon.version)
        new_version = self.prompt(msg, default=bump_minor(addon.version))
        addon.version = new_version
        self.app.stdout.write('Writing new version to addon.xml...')
        write_file('addon.xml', addon.to_xml_string())
        self.app.stdout.write(GREEN('OK') + '\n')
    
        # Check dependencies
        if len(addon.dependencies) > 1:
            msg = '[?] I see your addon has a few dependencies. Would you like to check for new versions?'
            if self.yes_no(msg):
                self.update_dependencies(addon, reponame)

        if os.path.exists('changelog.txt'):
            msg = '[?] I see you have a %s. Would you like to update it now?' % BLUE('changelog.txt')
            if self.yes_no(msg):
                check_call([os.getenv('EDITOR'), 'changelog.txt'])

        # Final ok
        msg = '[?] I\'m ready to commit the changes and tag the release. Should we continue?'
        self.exit_if_negative(msg)

        lines = [
            '',
            '.. Release %s..' % BLUE(addon.version),
            '',
        ]
        self.app.stdout.write('\n'.join(lines) + '\n')
    
        # git add -A .
        self.app.stdout.write('Adding local changes to staging...')
        check_call('git add -A', shell=True)
        self.app.stdout.write(GREEN('OK') + '\n')

        # write commit message
        self.app.stdout.write('Creating commit...')
        check_call('git commit -m "[xam-release-script] creating release for'
                   ' version %s"' % new_version, shell=True)
        self.app.stdout.write(GREEN('OK') + '\n')

        # git tag release
        self.app.stdout.write('Tagging release...')
        check_call('git tag -a "%s" -m "%s v%s"' % (new_version, reponame,
                                                         new_version), shell=True)
        self.app.stdout.write(GREEN('OK') + '\n')

        # git push
        self.app.stdout.write('Pushing commit and tags to remote...')
        check_call('git push && git push --tags', shell=True)
        self.app.stdout.write(GREEN('OK') + '\n')

        msg = GREEN('Congrats. Release was successful.')
        self.app.stdout.write('\n' + msg + '\n')

        # attempt to guess public repo URL if using github...  push_url = '<INSERT_REPO_URL>'
        output = check_output('git remote -v | grep origin | grep push | cut -f1'
                              ' -d" "', shell=True)
        match = re.match(r'origin\tgit@github.com:(.+?\.git)', output)
        push_url = RED('<YOUR REPO URL>')
        if match is not None:
            push_url = 'git://github.com/%s' % match.group(1)

        # print email
        output = generate_email_output(addon.id, addon.version, push_url, addon.version,
                    reponame.lower())
        self.app.stdout.write('\n'.join(output))



'''
XBMC Addon Manager
------------------

A CLI utility for searching and listing XBMC Addon Repositories.

Setup
`````

::

    $ pip install xam
    $ xam --help

Links
`````

* `website <http://github.com/jbeluch/xam/>`_

'''
from setuptools import setup, find_packages

def get_requires():
    '''If python > 2.7, argparse and OrderedDict will be included. Otherwsise
    we need external packages.
    '''
    requires = ['requests']
    try:
        import argparse
    except ImportError:
        requires.append('argparse')

    try:
        from collections import OrderedDict
    except ImportError:
        requires.append('collective.ordereddict')
    return requires


setup(
    name='xam',
    version='0.2.1',
    url='http://github.com/jbeluch/xam/',
    license='BSD',
    author='Jonathan Beluch',
    author_email='web@jonathanbeluch.com',
    description='A utility for listing, searching and viewing source code for '
                'XBMC addons.',
    long_description=__doc__,
    packages=find_packages(),
    platforms='any',
    install_requires=get_requires(),
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Environment :: Console',
        'Topic :: Utilities',
    ],
    entry_points={
        #'console_scripts': [
            #'xam = xam.cli:main'
        #]
        'console_scripts': [
            'xam = xam.main:main'
            ],
        'xam': [
            'all = xam.cli:ListAddons',
            'info = xam.cli:ShowAddonInfo',
            'depends = xam.cli:ShowDependentAddons',
            'get = xam.cli:GetAddon',
            'search = xam.cli:SearchAddons',
            'release = xam.cli.release:ReleaseAddon',
        ]
    },

)
